#!/usr/bin/env python3

import argparse
import re
import time
import logging
from random import randint
from time import sleep

import requests
from bs4 import BeautifulSoup


def getstatus(session):
    headers_status = {'User-Agent': user_agent}
    response = session.get(url_status, headers=headers_status)
    return response


def extenddata(session, getresponse):
    session.headers.update({'User-Agent': user_agent})
    data = {'code': upsell_code, 'zoneId': upsell_zoneid, 'subscriptionId': upsell_subid,
            'eventType': 'DATA', 'brand': 'SALT', 'lang': 'FR'}
    response = session.post(url_upsell, data=data, cookies=getresponse.cookies)
    return response


if __name__ == "__main__":
    start_time = time.perf_counter()
    logging.basicConfig(format='%(asctime)s,%(msecs)d %(name)s %(levelname)s %(message)s',
                        datefmt='%H:%M:%S',
                        level=logging.DEBUG,
                        handlers=[
                            logging.FileHandler("salt_auto_extend.log"),
                            logging.StreamHandler()
                            ]
                        )
    parser = argparse.ArgumentParser(description="Salt Multisurf Cap Auto Extend")
    parser.add_argument('-e', '--extend', type=int,
                        help="Number of +10GB extend the script will try to do. If not set, the amount will be automatically calculated.")
    parser.add_argument('-f', '--force', action='store_true', help="Try to force extend even if button not available.")
    parser.add_argument('-s', '--simulate', action='store_true', help="Simulate POST requests but don't send them.")
    parser.add_argument('-t', '--threshold', type=int, default=25,
                        help="the value below which the script will try to extend.")
    parser.add_argument('--min-wait', type=int, default=3, help="Minimum time wait before each extend attemps")
    parser.add_argument('--max-wait', type=int, default=10, help="Maxmimum time wait before each extend attemps")
    args = parser.parse_args()

    logging.info("Salt Multisurf Cap Auto Extend start")

    # Traitement des arguments CLI
    # Option --simulate
    if args.simulate:
        logging.info("Option simulate (-s) enabled")
        simulate_extends = True
    else:
        simulate_extends = False

    if args.force:
        logging.info("Option force (-f) enabled")
        force_extends = True
    else:
        force_extends = False

    # Option --extend
    extends_qty = 1
    if args.extend is None:
        calculate_extends = True
    else:
        logging.info("Option extend (-e) enabled")
        calculate_extends = False
        extends_qty = args.extend

    # Option --threshold
    extend_percent_threshold = args.threshold
    if extend_percent_threshold is not None:
        logging.info("Option threshold (-t) enabled")

    # Options --min-wait and --max-wait
    min_wait = args.min_wait
    max_wait = args.max_wait

    # 0. Prepare data
    user_agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:70.0) Gecko/20100101 Firefox/70.0'
    url_status = "http://costcontrol.salt.ch/status"
    url_upsell = "http://costcontrol.salt.ch/activateUpsell"
    upsell_code = "U30S10GNATPOSFREE"
    upsell_zoneid = "171"
    upsell_lang = 'FR'
    zone_anchor = "DATA_171_tab"
    regex_avail_vol = re.compile(r'Volume disponible : (\d+(?:\.\d+)?)(?: (.B))*')
    regex_init_val = re.compile(r'(\d+(?:\.\d+)?)(?: (.B))*')
    regex_init_perc = re.compile(r'(\d+)%.*')
    extend_validation_str = "Vous avez activé  ’10GB’ pour la zone Données en Suisse."

    session = requests.session()
    # 1. Get page
    response = getstatus(session)
    cookies = response.cookies
    html = response.text
    # 2. Parse page
    soup = BeautifulSoup(html, 'lxml')

    # 2.1 Check Subscription ID
    upsell_subid = soup.find('input', {'name': 'subscriptionId'}).get('value')
    logging.debug("Subscription ID: %s " % upsell_subid)

    # 2.2 Check Button Availability
    btAdd10 = soup.find("input", {'id': upsell_code})
    btAvailable = btAdd10 is not None
    if btAvailable:
        logging.info("+10GB Extend Button Available")
    else:
        logging.info("+10GB Extend Button Unavailable")

    # 2.2 Check remaining quota
    zone = soup.find("div", {'id': zone_anchor})
    remainder = zone.find(text=' counters ').find_next_sibling('div').find("div").find("h4").get_text().strip()
    avail_reg_match = regex_avail_vol.match(remainder)
    avail_reg_value = float(avail_reg_match.group(1))
    avail_reg_unit = avail_reg_match.group(2) if avail_reg_value > 0 else 'B'
    logging.info('Remaining quota : %.2f %s' % (avail_reg_value, avail_reg_unit))

    # 5. Extend
    # TODO: Calculate MB/GB/ETC
    if avail_reg_value < extend_percent_threshold:
        logging.info("%.2f is below the %d threshold. Extend is required" % (avail_reg_value, extend_percent_threshold))
        # TODO: Calculate number of extends needed
        extend_needed = extends_qty if not calculate_extends else 1
        logging.info("Number of extends to do : %d " % extend_needed)

        if btAvailable or force_extends:
            for i in range(extend_needed):
                if not simulate_extends:
                    logging.info("Extend %d:" % (i + 1))
                    extend = extenddata(session, response)
                    response_soup = BeautifulSoup(extend.text, 'lxml')
                    allparagraphs = response_soup.find_all('p')
                    if extend.status_code == 200 and extend_validation_str in allparagraphs[0].get_text().strip():
                        logging.info("Extend successful")
                    else:
                        logging.error("Request status: %s" % extend.status_code)
                        logging.error("An error occurred during extend:")
                        logging.error("Error :")
                        logging.error(allparagraphs[0].get_text().strip())
                else:
                    logging.warning("%d: This is a simulation, extend request was not sent" % (i + 1))
                if i < extend_needed - 1:
                    seconds = randint(min_wait, max_wait)
                    logging.info("Waiting %ds before next request" % seconds)
                    sleep(seconds)
        else:
            logging.warning("Extend Button not available, won't extend")
    else:
        logging.info("Total %.2f is above threshold (%d GB). Won't extend yet" % (avail_reg_value,
                                                                                  extend_percent_threshold))

    logging.info("--- Execution time: %s seconds ---" % (time.perf_counter() - start_time))
